class Strings(object):

    APP_NAME = "Omnilog"
    APP_RESTARTING = "Restarting app ..."
    APP_SHUTDOWN = "Shuting down app ..."
    CONFIG_ERROR = "Config error , check config. Shutting down ..."
    CONFIG_CHANGE = "Config changes detected ... setting new config."
    CONFIG_FILE_ERROR = "Cannot load config."
    CONFIG_CHANGES = "Config changes detected ... setting new config."
    SSH_ERROR = "SSH error , check config."
    KEYBOARD_INTERRUPTION = "keyboard interruption detected."
    UNHANDLED_EXCEPTION = "UNHANDLED EXCEPTION .... shuting down...."
    SUB_SYSTEM_START = "Starting subsystem"
    IO_ERROR = "IO error detected.Shutting down."
    LOG_PROCESSED = "Log processed."
    IPC_RECEIVED = " - IPC - Received "
    PARSER_VALID_LOG_REACHED = "Valid log reached, passing it to queue."
    WEBPANEL_PUBLISH = "Valid log reached, passing it to queue."
    SKELETON_DIR_CREATED = "Skeleton created in your home dir ... shutting down ..."
    SKELETON_DIR = "Skeleton exists, aborting ..."
    SKELETON_URL_ERROR = "Cannot contact external resources for skeleton ... go github."



